@@ -29,6 +29,7 @@ import re
 import tempfile
 import time
 import logging
+import pwd
 from cStringIO import StringIO
 
 from ganeti import utils
@@ -36,76 +37,226 @@ from ganeti import constants
 from ganeti import errors
 from ganeti import serializer
 from ganeti import objects
+from ganeti import uidpool
+from ganeti import ssconf
 from ganeti.hypervisor import hv_base
 
 
 class KVMHypervisor(hv_base.BaseHypervisor):
   """KVM hypervisor interface"""
+  CAN_MIGRATE = True
 
   _ROOT_DIR = constants.RUN_GANETI_DIR + "/kvm-hypervisor"
   _PIDS_DIR = _ROOT_DIR + "/pid" # contains live instances pids
+  _UIDS_DIR = _ROOT_DIR + "/uid" # contains instances reserved uids
   _CTRL_DIR = _ROOT_DIR + "/ctrl" # contains instances control sockets
   _CONF_DIR = _ROOT_DIR + "/conf" # contains instances startup data
-  _DIRS = [_ROOT_DIR, _PIDS_DIR, _CTRL_DIR, _CONF_DIR]
+  _DIRS = [_ROOT_DIR, _PIDS_DIR, _UIDS_DIR, _CTRL_DIR, _CONF_DIR]
 
-  PARAMETERS = [
-    constants.HV_KERNEL_PATH,
-    constants.HV_INITRD_PATH,
-    constants.HV_ROOT_PATH,
-    constants.HV_KERNEL_ARGS,
-    constants.HV_ACPI,
-    constants.HV_SERIAL_CONSOLE,
-    constants.HV_VNC_BIND_ADDRESS,
-    constants.HV_VNC_TLS,
-    constants.HV_VNC_X509,
-    constants.HV_VNC_X509_VERIFY,
-    constants.HV_CDROM_IMAGE_PATH,
-    constants.HV_BOOT_ORDER,
-    constants.HV_NIC_TYPE,
-    constants.HV_DISK_TYPE,
-    constants.HV_USB_MOUSE,
-    ]
+  PARAMETERS = {
+    constants.HV_KERNEL_PATH: hv_base.OPT_FILE_CHECK,
+    constants.HV_INITRD_PATH: hv_base.OPT_FILE_CHECK,
+    constants.HV_ROOT_PATH: hv_base.NO_CHECK,
+    constants.HV_KERNEL_ARGS: hv_base.NO_CHECK,
+    constants.HV_ACPI: hv_base.NO_CHECK,
+    constants.HV_SERIAL_CONSOLE: hv_base.NO_CHECK,
+    constants.HV_VNC_BIND_ADDRESS:
+      (False, lambda x: (utils.IsValidIP(x) or utils.IsNormAbsPath(x)),
+       "the VNC bind address must be either a valid IP address or an absolute"
+       " pathname", None, None),
+    constants.HV_VNC_TLS: hv_base.NO_CHECK,
+    constants.HV_VNC_X509: hv_base.OPT_DIR_CHECK,
+    constants.HV_VNC_X509_VERIFY: hv_base.NO_CHECK,
+    constants.HV_VNC_PASSWORD_FILE: hv_base.OPT_FILE_CHECK,
+    constants.HV_CDROM_IMAGE_PATH: hv_base.OPT_FILE_CHECK,
+    constants.HV_BOOT_ORDER:
+      hv_base.ParamInSet(True, constants.HT_KVM_VALID_BO_TYPES),
+    constants.HV_NIC_TYPE:
+      hv_base.ParamInSet(True, constants.HT_KVM_VALID_NIC_TYPES),
+    constants.HV_DISK_TYPE:
+      hv_base.ParamInSet(True, constants.HT_KVM_VALID_DISK_TYPES),
+    constants.HV_USB_MOUSE:
+      hv_base.ParamInSet(False, constants.HT_KVM_VALID_MOUSE_TYPES),
+    constants.HV_MIGRATION_PORT: hv_base.NET_PORT_CHECK,
+    constants.HV_USE_LOCALTIME: hv_base.NO_CHECK,
+    constants.HV_DISK_CACHE:
+      hv_base.ParamInSet(True, constants.HT_VALID_CACHE_TYPES),
+    constants.HV_SECURITY_MODEL:
+      hv_base.ParamInSet(True, constants.HT_KVM_VALID_SM_TYPES),
+    constants.HV_SECURITY_DOMAIN: hv_base.NO_CHECK,
+    constants.HV_KVM_FLAG:
+      hv_base.ParamInSet(False, constants.HT_KVM_FLAG_VALUES),
+    }
 
   _MIGRATION_STATUS_RE = re.compile('Migration\s+status:\s+(\w+)',
                                     re.M | re.I)
+  _MIGRATION_INFO_MAX_BAD_ANSWERS = 5
+  _MIGRATION_INFO_RETRY_DELAY = 2
+
+  _KVM_NETWORK_SCRIPT = constants.SYSCONFDIR + "/ganeti/kvm-vif-bridge"
+
+  ANCILLARY_FILES = [
+    _KVM_NETWORK_SCRIPT,
+    ]
 
   def __init__(self):
     hv_base.BaseHypervisor.__init__(self)
     # Let's make sure the directories we need exist, even if the RUN_DIR lives
     # in a tmpfs filesystem or has been otherwise wiped out.
-    dirs = [(dir, constants.RUN_DIRS_MODE) for dir in self._DIRS]
+    dirs = [(dname, constants.RUN_DIRS_MODE) for dname in self._DIRS]
     utils.EnsureDirs(dirs)
 
-  def _InstancePidAlive(self, instance_name):
-    """Returns the instance pid and pidfile
+  @classmethod
+  def _InstancePidFile(cls, instance_name):
+    """Returns the instance pidfile.
+
+    """
+    return utils.PathJoin(cls._PIDS_DIR, instance_name)
+
+  @classmethod
+  def _InstanceUidFile(cls, instance_name):
+    """Returns the instance uidfile.
+
+    """
+    return utils.PathJoin(cls._UIDS_DIR, instance_name)
+
+  @classmethod
+  def _InstancePidInfo(cls, pid):
+    """Check pid file for instance information.
+
+    Check that a pid file is associated with an instance, and retrieve
+    information from its command line.
+
+    @type pid: string or int
+    @param pid: process id of the instance to check
+    @rtype: tuple
+    @return: (instance_name, memory, vcpus)
+    @raise errors.HypervisorError: when an instance cannot be found
 
     """
-    pidfile = "%s/%s" % (self._PIDS_DIR, instance_name)
-    pid = utils.ReadPidFile(pidfile)
     alive = utils.IsProcessAlive(pid)
+    if not alive:
+      raise errors.HypervisorError("Cannot get info for pid %s" % pid)
+
+    cmdline_file = utils.PathJoin("/proc", str(pid), "cmdline")
+    try:
+      cmdline = utils.ReadFile(cmdline_file)
+    except EnvironmentError, err:
+      raise errors.HypervisorError("Can't open cmdline file for pid %s: %s" %
+                                   (pid, err))
+
+    instance = None
+    memory = 0
+    vcpus = 0
+
+    arg_list = cmdline.split('\x00')
+    while arg_list:
+      arg =  arg_list.pop(0)
+      if arg == "-name":
+        instance = arg_list.pop(0)
+      elif arg == "-m":
+        memory = int(arg_list.pop(0))
+      elif arg == "-smp":
+        vcpus = int(arg_list.pop(0))
+
+    if instance is None:
+      raise errors.HypervisorError("Pid %s doesn't contain a ganeti kvm"
+                                   " instance" % pid)
+
+    return (instance, memory, vcpus)
+
+  def _InstancePidAlive(self, instance_name):
+    """Returns the instance pidfile, pid, and liveness.
+
+    @type instance_name: string
+    @param instance_name: instance name
+    @rtype: tuple
+    @return: (pid file name, pid, liveness)
+
+    """
+    pidfile = self._InstancePidFile(instance_name)
+    pid = utils.ReadPidFile(pidfile)
+
+    alive = False
+    try:
+      cmd_instance = self._InstancePidInfo(pid)[0]
+      alive = (cmd_instance == instance_name)
+    except errors.HypervisorError:
+      pass
 
     return (pidfile, pid, alive)
 
+  def _CheckDown(self, instance_name):
+    """Raises an error unless the given instance is down.
+
+    """
+    alive = self._InstancePidAlive(instance_name)[2]
+    if alive:
+      raise errors.HypervisorError("Failed to start instance %s: %s" %
+                                   (instance_name, "already running"))
+
   @classmethod
   def _InstanceMonitor(cls, instance_name):
     """Returns the instance monitor socket name
 
     """
-    return '%s/%s.monitor' % (cls._CTRL_DIR, instance_name)
+    return utils.PathJoin(cls._CTRL_DIR, "%s.monitor" % instance_name)
 
   @classmethod
   def _InstanceSerial(cls, instance_name):
     """Returns the instance serial socket name
 
     """
-    return '%s/%s.serial' % (cls._CTRL_DIR, instance_name)
+    return utils.PathJoin(cls._CTRL_DIR, "%s.serial" % instance_name)
+
+  @staticmethod
+  def _SocatUnixConsoleParams():
+    """Returns the correct parameters for socat
+
+    If we have a new-enough socat we can use raw mode with an escape character.
+
+    """
+    if constants.SOCAT_USE_ESCAPE:
+      return "raw,echo=0,escape=%s" % constants.SOCAT_ESCAPE_CODE
+    else:
+      return "echo=0,icanon=0"
 
   @classmethod
   def _InstanceKVMRuntime(cls, instance_name):
     """Returns the instance KVM runtime filename
 
     """
-    return '%s/%s.runtime' % (cls._CONF_DIR, instance_name)
+    return utils.PathJoin(cls._CONF_DIR, "%s.runtime" % instance_name)
+
+  @classmethod
+  def _TryReadUidFile(cls, uid_file):
+    """Try to read a uid file
+
+    """
+    if os.path.exists(uid_file):
+      try:
+        uid = int(utils.ReadFile(uid_file))
+        return uid
+      except EnvironmentError:
+        logging.warning("Can't read uid file", exc_info=True)
+      except (TypeError, ValueError):
+        logging.warning("Can't parse uid file contents", exc_info=True)
+    return None
+
+  @classmethod
+  def _RemoveInstanceRuntimeFiles(cls, pidfile, instance_name):
+    """Removes an instance's rutime sockets/files.
+
+    """
+    utils.RemoveFile(pidfile)
+    utils.RemoveFile(cls._InstanceMonitor(instance_name))
+    utils.RemoveFile(cls._InstanceSerial(instance_name))
+    utils.RemoveFile(cls._InstanceKVMRuntime(instance_name))
+    uid_file = cls._InstanceUidFile(instance_name)
+    uid = cls._TryReadUidFile(uid_file)
+    utils.RemoveFile(uid_file)
+    if uid is not None:
+      uidpool.ReleaseUid(uid)
 
   def _WriteNetScript(self, instance, seq, nic):
     """Write a script to connect a net interface to the proper bridge.
@@ -125,26 +276,59 @@ class KVMHypervisor(hv_base.BaseHypervisor):
     script = StringIO()
     script.write("#!/bin/sh\n")
     script.write("# this is autogenerated by Ganeti, please do not edit\n#\n")
+    script.write("PATH=$PATH:/sbin:/usr/sbin\n")
     script.write("export INSTANCE=%s\n" % instance.name)
     script.write("export MAC=%s\n" % nic.mac)
-    script.write("export IP=%s\n" % nic.ip)
-    script.write("export BRIDGE=%s\n" % nic.bridge)
+    if nic.ip:
+      script.write("export IP=%s\n" % nic.ip)
+    script.write("export MODE=%s\n" % nic.nicparams[constants.NIC_MODE])
+    if nic.nicparams[constants.NIC_LINK]:
+      script.write("export LINK=%s\n" % nic.nicparams[constants.NIC_LINK])
+    if nic.nicparams[constants.NIC_MODE] == constants.NIC_MODE_BRIDGED:
+      script.write("export BRIDGE=%s\n" % nic.nicparams[constants.NIC_LINK])
     script.write("export INTERFACE=$1\n")
+    if instance.tags:
+      script.write("export TAGS=\"%s\"\n" % " ".join(instance.tags))
     # TODO: make this configurable at ./configure time
-    script.write("if [ -x /etc/ganeti/kvm-vif-bridge ]; then\n")
+    script.write("if [ -x '%s' ]; then\n" % self._KVM_NETWORK_SCRIPT)
     script.write("  # Execute the user-specific vif file\n")
-    script.write("  /etc/ganeti/kvm-vif-bridge\n")
+    script.write("  %s\n" % self._KVM_NETWORK_SCRIPT)
     script.write("else\n")
-    script.write("  # Connect the interface to the bridge\n")
-    script.write("  /sbin/ifconfig $INTERFACE 0.0.0.0 up\n")
-    script.write("  /usr/sbin/brctl addif $BRIDGE $INTERFACE\n")
+    script.write("  ifconfig $INTERFACE 0.0.0.0 up\n")
+    if nic.nicparams[constants.NIC_MODE] == constants.NIC_MODE_BRIDGED:
+      script.write("  # Connect the interface to the bridge\n")
+      script.write("  brctl addif $BRIDGE $INTERFACE\n")
+    elif nic.nicparams[constants.NIC_MODE] == constants.NIC_MODE_ROUTED:
+      if not nic.ip:
+        raise errors.HypervisorError("nic/%d is routed, but has no ip." % seq)
+      script.write("  # Route traffic targeted at the IP to the interface\n")
+      if nic.nicparams[constants.NIC_LINK]:
+        script.write("  while ip rule del dev $INTERFACE; do :; done\n")
+        script.write("  ip rule add dev $INTERFACE table $LINK\n")
+        script.write("  ip route replace $IP table $LINK proto static"
+                     " dev $INTERFACE\n")
+      else:
+        script.write("  ip route replace $IP proto static"
+                     " dev $INTERFACE\n")
+      interface_v4_conf = "/proc/sys/net/ipv4/conf/$INTERFACE"
+      interface_v6_conf = "/proc/sys/net/ipv6/conf/$INTERFACE"
+      script.write("  if [ -d %s ]; then\n" % interface_v4_conf)
+      script.write("    echo 1 > %s/proxy_arp\n" % interface_v4_conf)
+      script.write("    echo 1 > %s/forwarding\n" % interface_v4_conf)
+      script.write("  fi\n")
+      script.write("  if [ -d %s ]; then\n" % interface_v6_conf)
+      script.write("    echo 1 > %s/proxy_ndp\n" % interface_v6_conf)
+      script.write("    echo 1 > %s/forwarding\n" % interface_v6_conf)
+      script.write("  fi\n")
     script.write("fi\n\n")
     # As much as we'd like to put this in our _ROOT_DIR, that will happen to be
     # mounted noexec sometimes, so we'll have to find another place.
     (tmpfd, tmpfile_name) = tempfile.mkstemp()
     tmpfile = os.fdopen(tmpfd, 'w')
-    tmpfile.write(script.getvalue())
-    tmpfile.close()
+    try:
+      tmpfile.write(script.getvalue())
+    finally:
+      tmpfile.close()
     os.chmod(tmpfile_name, 0755)
     return tmpfile_name
 
@@ -157,47 +341,27 @@ class KVMHypervisor(hv_base.BaseHypervisor):
     """
     result = []
     for name in os.listdir(self._PIDS_DIR):
-      filename = "%s/%s" % (self._PIDS_DIR, name)
-      if utils.IsProcessAlive(utils.ReadPidFile(filename)):
+      if self._InstancePidAlive(name)[2]:
         result.append(name)
     return result
 
   def GetInstanceInfo(self, instance_name):
     """Get instance properties.
 
+    @type instance_name: string
     @param instance_name: the instance name
-
-    @return: tuple (name, id, memory, vcpus, stat, times)
+    @rtype: tuple of strings
+    @return: (name, id, memory, vcpus, stat, times)
 
     """
-    pidfile, pid, alive = self._InstancePidAlive(instance_name)
+    _, pid, alive = self._InstancePidAlive(instance_name)
     if not alive:
       return None
 
-    cmdline_file = "/proc/%s/cmdline" % pid
-    try:
-      fh = open(cmdline_file, 'r')
-      try:
-        cmdline = fh.read()
-      finally:
-        fh.close()
-    except EnvironmentError, err:
-      raise errors.HypervisorError("Failed to list instance %s: %s" %
-                                   (instance_name, err))
-
-    memory = 0
-    vcpus = 0
+    _, memory, vcpus = self._InstancePidInfo(pid)
     stat = "---b-"
     times = "0"
 
-    arg_list = cmdline.split('\x00')
-    while arg_list:
-      arg =  arg_list.pop(0)
-      if arg == '-m':
-        memory = int(arg_list.pop(0))
-      elif arg == '-smp':
-        vcpus = int(arg_list.pop(0))
-
     return (instance_name, pid, memory, vcpus, stat, times)
 
   def GetAllInstancesInfo(self):
@@ -208,29 +372,26 @@ class KVMHypervisor(hv_base.BaseHypervisor):
     """
     data = []
     for name in os.listdir(self._PIDS_DIR):
-      filename = "%s/%s" % (self._PIDS_DIR, name)
-      if utils.IsProcessAlive(utils.ReadPidFile(filename)):
-        try:
-          info = self.GetInstanceInfo(name)
-        except errors.HypervisorError, err:
-          continue
-        if info:
-          data.append(info)
-
+      try:
+        info = self.GetInstanceInfo(name)
+      except errors.HypervisorError:
+        continue
+      if info:
+        data.append(info)
     return data
 
   def _GenerateKVMRuntime(self, instance, block_devices):
     """Generate KVM information to start an instance.
 
     """
-    pidfile, pid, alive = self._InstancePidAlive(instance.name)
+    pidfile  = self._InstancePidFile(instance.name)
     kvm = constants.KVM_PATH
     kvm_cmd = [kvm]
+    # used just by the vnc server, if enabled
+    kvm_cmd.extend(['-name', instance.name])
     kvm_cmd.extend(['-m', instance.beparams[constants.BE_MEMORY]])
     kvm_cmd.extend(['-smp', instance.beparams[constants.BE_VCPUS]])
     kvm_cmd.extend(['-pidfile', pidfile])
-    # used just by the vnc server, if enabled
-    kvm_cmd.extend(['-name', instance.name])
     kvm_cmd.extend(['-daemonize'])
     if not instance.hvparams[constants.HV_ACPI]:
       kvm_cmd.extend(['-no-acpi'])
@@ -240,6 +401,11 @@ class KVMHypervisor(hv_base.BaseHypervisor):
     boot_cdrom = hvp[constants.HV_BOOT_ORDER] == constants.HT_BO_CDROM
     boot_network = hvp[constants.HV_BOOT_ORDER] == constants.HT_BO_NETWORK
 
+    if hvp[constants.HV_KVM_FLAG] == constants.HT_KVM_ENABLED:
+      kvm_cmd.extend(["-enable-kvm"])
+    elif hvp[constants.HV_KVM_FLAG] == constants.HT_KVM_DISABLED:
+      kvm_cmd.extend(["-disable-kvm"])
+
     if boot_network:
       kvm_cmd.extend(['-boot', 'n'])
 
@@ -248,6 +414,12 @@ class KVMHypervisor(hv_base.BaseHypervisor):
       if_val = ',if=virtio'
     else:
       if_val = ',if=%s' % disk_type
+    # Cache mode
+    disk_cache = hvp[constants.HV_DISK_CACHE]
+    if disk_cache != constants.HT_CACHE_DEFAULT:
+      cache_val = ",cache=%s" % disk_cache
+    else:
+      cache_val = ""
     for cfdev, dev_path in block_devices:
       if cfdev.mode != constants.DISK_RDWR:
         raise errors.HypervisorError("Instance has read-only disks which"
@@ -255,13 +427,17 @@ class KVMHypervisor(hv_base.BaseHypervisor):
       # TODO: handle FD_LOOP and FD_BLKTAP (?)
       if boot_disk:
         kvm_cmd.extend(['-boot', 'c'])
-        boot_val = ',boot=on'
+        if disk_type != constants.HT_DISK_IDE:
+          boot_val = ',boot=on'
+        else:
+          boot_val = ''
         # We only boot from the first disk
         boot_disk = False
       else:
         boot_val = ''
 
-      drive_val = 'file=%s,format=raw%s%s' % (dev_path, if_val, boot_val)
+      drive_val = 'file=%s,format=raw%s%s%s' % (dev_path, if_val, boot_val,
+                                                cache_val)
       kvm_cmd.extend(['-drive', drive_val])
 
     iso_image = hvp[constants.HV_CDROM_IMAGE_PATH]
@@ -269,9 +445,14 @@ class KVMHypervisor(hv_base.BaseHypervisor):
       options = ',format=raw,media=cdrom'
       if boot_cdrom:
         kvm_cmd.extend(['-boot', 'd'])
-        options = '%s,boot=on' % options
+        if disk_type != constants.HT_DISK_IDE:
+          options = '%s,boot=on' % options
       else:
-        options = '%s,if=virtio' % options
+        if disk_type == constants.HT_DISK_PARAVIRTUAL:
+          if_val = ',if=virtio'
+        else:
+          if_val = ',if=%s' % disk_type
+        options = '%s%s' % (options, if_val)
       drive_val = 'file=%s%s' % (iso_image, options)
       kvm_cmd.extend(['-drive', drive_val])
 
@@ -292,7 +473,6 @@ class KVMHypervisor(hv_base.BaseHypervisor):
       kvm_cmd.extend(['-usb'])
       kvm_cmd.extend(['-usbdevice', mouse_type])
 
-    # FIXME: handle vnc password
     vnc_bind_address = hvp[constants.HV_VNC_BIND_ADDRESS]
     if vnc_bind_address:
       if utils.IsValidIP(vnc_bind_address):
@@ -304,9 +484,8 @@ class KVMHypervisor(hv_base.BaseHypervisor):
             vnc_arg = '%s:%d' % (vnc_bind_address, display)
         else:
           logging.error("Network port is not a valid VNC display (%d < %d)."
-                        " Not starting VNC" %
-                        (instance.network_port,
-                         constants.VNC_BASE_PORT))
+                        " Not starting VNC", instance.network_port,
+                        constants.VNC_BASE_PORT)
           vnc_arg = 'none'
 
         # Only allow tls and other option when not binding to a file, for now.
@@ -320,17 +499,24 @@ class KVMHypervisor(hv_base.BaseHypervisor):
           elif hvp[constants.HV_VNC_X509]:
             vnc_append = '%s,x509=%s' % (vnc_append,
                                          hvp[constants.HV_VNC_X509])
+        if hvp[constants.HV_VNC_PASSWORD_FILE]:
+          vnc_append = '%s,password' % vnc_append
+
         vnc_arg = '%s%s' % (vnc_arg, vnc_append)
 
       else:
         vnc_arg = 'unix:%s/%s.vnc' % (vnc_bind_address, instance.name)
 
       kvm_cmd.extend(['-vnc', vnc_arg])
+
+      # Also add a tablet USB device to act as a mouse
+      # This solves various mouse alignment issues
+      kvm_cmd.extend(['-usbdevice', 'tablet'])
     else:
       kvm_cmd.extend(['-nographic'])
 
-    monitor_dev = 'unix:%s,server,nowait' % \
-      self._InstanceMonitor(instance.name)
+    monitor_dev = ("unix:%s,server,nowait" %
+                   self._InstanceMonitor(instance.name))
     kvm_cmd.extend(['-monitor', monitor_dev])
     if hvp[constants.HV_SERIAL_CONSOLE]:
       serial_dev = ('unix:%s,server,nowait' %
@@ -339,6 +525,9 @@ class KVMHypervisor(hv_base.BaseHypervisor):
     else:
       kvm_cmd.extend(['-serial', 'none'])
 
+    if hvp[constants.HV_USE_LOCALTIME]:
+      kvm_cmd.extend(['-localtime'])
+
     # Save the current instance nics, but defer their expansion as parameters,
     # as we'll need to generate executable temp files for them.
     kvm_nics = instance.nics
@@ -386,6 +575,22 @@ class KVMHypervisor(hv_base.BaseHypervisor):
     kvm_nics = [objects.NIC.FromDict(snic) for snic in serialized_nics]
     return (kvm_cmd, kvm_nics, hvparams)
 
+  def _RunKVMCmd(self, name, kvm_cmd):
+    """Run the KVM cmd and check for errors
+
+    @type name: string
+    @param name: instance name
+    @type kvm_cmd: list of strings
+    @param kvm_cmd: runcmd input for kvm
+
+    """
+    result = utils.RunCmd(kvm_cmd)
+    if result.failed:
+      raise errors.HypervisorError("Failed to start instance %s: %s (%s)" %
+                                   (name, result.fail_reason, result.output))
+    if not self._InstancePidAlive(name)[2]:
+      raise errors.HypervisorError("Failed to start instance %s" % name)
+
   def _ExecuteKVMRuntime(self, instance, kvm_runtime, incoming=None):
     """Execute a KVM cmd, after completing it with some last minute data
 
@@ -393,15 +598,18 @@ class KVMHypervisor(hv_base.BaseHypervisor):
     @param incoming: (target_host_ip, port)
 
     """
-    pidfile, pid, alive = self._InstancePidAlive(instance.name)
-    if alive:
-      raise errors.HypervisorError("Failed to start instance %s: %s" %
-                                   (instance.name, "already running"))
+    hvp = instance.hvparams
+    name = instance.name
+    self._CheckDown(name)
 
     temp_files = []
 
     kvm_cmd, kvm_nics, hvparams = kvm_runtime
 
+    security_model = hvp[constants.HV_SECURITY_MODEL]
+    if security_model == constants.HT_SM_USER:
+      kvm_cmd.extend(["-runas", hvp[constants.HV_SECURITY_DOMAIN]])
+
     if not kvm_nics:
       kvm_cmd.extend(['-net', 'none'])
     else:
@@ -412,25 +620,46 @@ class KVMHypervisor(hv_base.BaseHypervisor):
         nic_model = "model=%s" % nic_type
 
       for nic_seq, nic in enumerate(kvm_nics):
-        nic_val = "nic,macaddr=%s,%s" % (nic.mac, nic_model)
+        nic_val = "nic,vlan=%s,macaddr=%s,%s" % (nic_seq, nic.mac, nic_model)
         script = self._WriteNetScript(instance, nic_seq, nic)
         kvm_cmd.extend(['-net', nic_val])
-        kvm_cmd.extend(['-net', 'tap,script=%s' % script])
+        kvm_cmd.extend(['-net', 'tap,vlan=%s,script=%s' % (nic_seq, script)])
         temp_files.append(script)
 
     if incoming:
       target, port = incoming
       kvm_cmd.extend(['-incoming', 'tcp:%s:%s' % (target, port)])
 
-    result = utils.RunCmd(kvm_cmd)
-    if result.failed:
-      raise errors.HypervisorError("Failed to start instance %s: %s (%s)" %
-                                   (instance.name, result.fail_reason,
-                                    result.output))
+    vnc_pwd_file = hvp[constants.HV_VNC_PASSWORD_FILE]
+    vnc_pwd = None
+    if vnc_pwd_file:
+      try:
+        vnc_pwd = utils.ReadFile(vnc_pwd_file)
+      except EnvironmentError, err:
+        raise errors.HypervisorError("Failed to open VNC password file %s: %s"
+                                     % (vnc_pwd_file, err))
 
-    if not utils.IsProcessAlive(utils.ReadPidFile(pidfile)):
-      raise errors.HypervisorError("Failed to start instance %s: %s" %
-                                   (instance.name))
+    if security_model == constants.HT_SM_POOL:
+      ss = ssconf.SimpleStore()
+      uid_pool = uidpool.ParseUidPool(ss.GetUidPool(), separator="\n")
+      all_uids = set(uidpool.ExpandUidPool(uid_pool))
+      uid = uidpool.RequestUnusedUid(all_uids)
+      try:
+        username = pwd.getpwuid(uid.GetUid()).pw_name
+        kvm_cmd.extend(["-runas", username])
+        self._RunKVMCmd(name, kvm_cmd)
+      except:
+        uidpool.ReleaseUid(uid)
+        raise
+      else:
+        uid.Unlock()
+        utils.WriteFile(self._InstanceUidFile(name), data=str(uid))
+    else:
+      self._RunKVMCmd(name, kvm_cmd)
+
+    if vnc_pwd:
+      change_cmd = 'change vnc password %s' % vnc_pwd
+      self._CallMonitorCommand(instance.name, change_cmd)
 
     for filename in temp_files:
       utils.RemoveFile(filename)
@@ -439,11 +668,7 @@ class KVMHypervisor(hv_base.BaseHypervisor):
     """Start an instance.
 
     """
-    pidfile, pid, alive = self._InstancePidAlive(instance.name)
-    if alive:
-      raise errors.HypervisorError("Failed to start instance %s: %s" %
-                                   (instance.name, "already running"))
-
+    self._CheckDown(instance.name)
     kvm_runtime = self._GenerateKVMRuntime(instance, block_devices)
     self._SaveKVMRuntime(instance, kvm_runtime)
     self._ExecuteKVMRuntime(instance, kvm_runtime)
@@ -466,39 +691,32 @@ class KVMHypervisor(hv_base.BaseHypervisor):
 
     return result
 
-  def _RetryInstancePowerdown(self, instance, pid, timeout=30):
-    """Wait for an instance  to power down.
-
-    """
-    # Wait up to $timeout seconds
-    end = time.time() + timeout
-    wait = 1
-    while time.time() < end and utils.IsProcessAlive(pid):
-      self._CallMonitorCommand(instance.name, 'system_powerdown')
-      time.sleep(wait)
-      # Make wait time longer for next try
-      if wait < 5:
-        wait *= 1.3
-
-  def StopInstance(self, instance, force=False):
+  def StopInstance(self, instance, force=False, retry=False, name=None):
     """Stop an instance.
 
     """
-    pidfile, pid, alive = self._InstancePidAlive(instance.name)
+    if name is not None and not force:
+      raise errors.HypervisorError("Cannot shutdown cleanly by name only")
+    if name is None:
+      name = instance.name
+      acpi = instance.hvparams[constants.HV_ACPI]
+    else:
+      acpi = False
+    _, pid, alive = self._InstancePidAlive(name)
     if pid > 0 and alive:
-      if force or not instance.hvparams[constants.HV_ACPI]:
+      if force or not acpi:
         utils.KillProcess(pid)
       else:
-        self._RetryInstancePowerdown(instance, pid)
+        self._CallMonitorCommand(name, 'system_powerdown')
 
-    if not utils.IsProcessAlive(pid):
-      utils.RemoveFile(pidfile)
-      utils.RemoveFile(self._InstanceMonitor(instance.name))
-      utils.RemoveFile(self._InstanceSerial(instance.name))
-      utils.RemoveFile(self._InstanceKVMRuntime(instance.name))
-      return True
-    else:
-      return False
+  def CleanupInstance(self, instance_name):
+    """Cleanup after a stopped instance
+
+    """
+    pidfile, pid, alive = self._InstancePidAlive(instance_name)
+    if pid > 0 and alive:
+      raise errors.HypervisorError("Cannot cleanup a live instance")
+    self._RemoveInstanceRuntimeFiles(pidfile, instance_name)
 
   def RebootInstance(self, instance):
     """Reboot an instance.
@@ -507,10 +725,10 @@ class KVMHypervisor(hv_base.BaseHypervisor):
     # For some reason if we do a 'send-key ctrl-alt-delete' to the control
     # socket the instance will stop, but now power up again. So we'll resort
     # to shutdown and restart.
-    pidfile, pid, alive = self._InstancePidAlive(instance.name)
+    _, _, alive = self._InstancePidAlive(instance.name)
     if not alive:
-      raise errors.HypervisorError("Failed to reboot instance %s: not running" %
-                                             (instance.name))
+      raise errors.HypervisorError("Failed to reboot instance %s:"
+                                   " not running" % instance.name)
     # StopInstance will delete the saved KVM runtime so:
     # ...first load it...
     kvm_runtime = self._LoadKVMRuntime(instance)
@@ -544,7 +762,7 @@ class KVMHypervisor(hv_base.BaseHypervisor):
 
     """
     kvm_runtime = self._LoadKVMRuntime(instance, serialized_runtime=info)
-    incoming_address = (target, constants.KVM_MIGRATION_PORT)
+    incoming_address = (target, instance.hvparams[constants.HV_MIGRATION_PORT])
     self._ExecuteKVMRuntime(instance, kvm_runtime, incoming=incoming_address)
 
   def FinalizeMigration(self, instance, info, success):
@@ -561,59 +779,72 @@ class KVMHypervisor(hv_base.BaseHypervisor):
     else:
       self.StopInstance(instance, force=True)
 
-  def MigrateInstance(self, instance_name, target, live):
+  def MigrateInstance(self, instance, target, live):
     """Migrate an instance to a target node.
 
     The migration will not be attempted if the instance is not
     currently running.
 
-    @type instance_name: string
-    @param instance_name: name of the instance to be migrated
+    @type instance: L{objects.Instance}
+    @param instance: the instance to be migrated
     @type target: string
     @param target: ip address of the target node
     @type live: boolean
     @param live: perform a live migration
 
     """
+    instance_name = instance.name
+    port = instance.hvparams[constants.HV_MIGRATION_PORT]
     pidfile, pid, alive = self._InstancePidAlive(instance_name)
     if not alive:
       raise errors.HypervisorError("Instance not running, cannot migrate")
 
+    if not utils.TcpPing(target, port, live_port_needed=True):
+      raise errors.HypervisorError("Remote host %s not listening on port"
+                                   " %s, cannot migrate" % (target, port))
+
     if not live:
       self._CallMonitorCommand(instance_name, 'stop')
 
-    migrate_command = ('migrate -d tcp:%s:%s' %
-                       (target, constants.KVM_MIGRATION_PORT))
+    migrate_command = 'migrate -d tcp:%s:%s' % (target, port)
     self._CallMonitorCommand(instance_name, migrate_command)
 
     info_command = 'info migrate'
     done = False
+    broken_answers = 0
     while not done:
       result = self._CallMonitorCommand(instance_name, info_command)
       match = self._MIGRATION_STATUS_RE.search(result.stdout)
       if not match:
-        raise errors.HypervisorError("Unknown 'info migrate' result: %s" %
-                                     result.stdout)
+        broken_answers += 1
+        if not result.stdout:
+          logging.info("KVM: empty 'info migrate' result")
+        else:
+          logging.warning("KVM: unknown 'info migrate' result: %s",
+                          result.stdout)
+        time.sleep(self._MIGRATION_INFO_RETRY_DELAY)
       else:
         status = match.group(1)
         if status == 'completed':
           done = True
         elif status == 'active':
-          time.sleep(2)
+          # reset the broken answers count
+          broken_answers = 0
+          time.sleep(self._MIGRATION_INFO_RETRY_DELAY)
         elif status == 'failed' or status == 'cancelled':
           if not live:
             self._CallMonitorCommand(instance_name, 'cont')
           raise errors.HypervisorError("Migration %s at the kvm level" %
                                        status)
         else:
-          logging.info("KVM: unknown migration status '%s'" % status)
-          time.sleep(2)
+          logging.warning("KVM: unknown migration status '%s'", status)
+          broken_answers += 1
+          time.sleep(self._MIGRATION_INFO_RETRY_DELAY)
+      if broken_answers >= self._MIGRATION_INFO_MAX_BAD_ANSWERS:
+        raise errors.HypervisorError("Too many 'info migrate' broken answers")
 
     utils.KillProcess(pid)
-    utils.RemoveFile(pidfile)
-    utils.RemoveFile(self._InstanceMonitor(instance_name))
-    utils.RemoveFile(self._InstanceSerial(instance_name))
-    utils.RemoveFile(self._InstanceKVMRuntime(instance_name))
+    self._RemoveInstanceRuntimeFiles(pidfile, instance_name)
 
   def GetNodeInfo(self):
     """Return information about the node.
@@ -634,15 +865,8 @@ class KVMHypervisor(hv_base.BaseHypervisor):
 
     """
     if hvparams[constants.HV_SERIAL_CONSOLE]:
-      # FIXME: The socat shell is not perfect. In particular the way we start
-      # it ctrl+c will close it, rather than being passed to the other end.
-      # On the other hand if we pass the option 'raw' (or ignbrk=1) there
-      # will be no way of exiting socat (except killing it from another shell)
-      # and ctrl+c doesn't work anyway, printing ^C rather than being
-      # interpreted by kvm. For now we'll leave it this way, which at least
-      # allows a minimal interaction and changes on the machine.
-      shell_command = ("%s STDIO,echo=0,icanon=0 UNIX-CONNECT:%s" %
-                       (constants.SOCAT_PATH,
+      shell_command = ("%s STDIO,%s UNIX-CONNECT:%s" %
+                       (constants.SOCAT_PATH, cls._SocatUnixConsoleParams(),
                         utils.ShellQuote(cls._InstanceSerial(instance.name))))
     else:
       shell_command = "echo 'No serial shell for instance %s'" % instance.name
@@ -684,109 +908,56 @@ class KVMHypervisor(hv_base.BaseHypervisor):
 
     kernel_path = hvparams[constants.HV_KERNEL_PATH]
     if kernel_path:
-      if not os.path.isabs(hvparams[constants.HV_KERNEL_PATH]):
-        raise errors.HypervisorError("The kernel path must be an absolute path"
-                                     ", if defined")
-
       if not hvparams[constants.HV_ROOT_PATH]:
-        raise errors.HypervisorError("Need a root partition for the instance"
-                                     ", if a kernel is defined")
+        raise errors.HypervisorError("Need a root partition for the instance,"
+                                     " if a kernel is defined")
 
-    if hvparams[constants.HV_INITRD_PATH]:
-      if not os.path.isabs(hvparams[constants.HV_INITRD_PATH]):
-        raise errors.HypervisorError("The initrd path must an absolute path"
-                                     ", if defined")
-
-    vnc_bind_address = hvparams[constants.HV_VNC_BIND_ADDRESS]
-    if vnc_bind_address:
-      if not utils.IsValidIP(vnc_bind_address):
-        if not os.path.isabs(vnc_bind_address):
-          raise errors.HypervisorError("The VNC bind address must be either"
-                                       " a valid IP address or an absolute"
-                                       " pathname. '%s' given" %
-                                       vnc_bind_address)
-
-    if hvparams[constants.HV_VNC_X509_VERIFY] and \
-      not hvparams[constants.HV_VNC_X509]:
-        raise errors.HypervisorError("%s must be defined, if %s is" %
-                                     (constants.HV_VNC_X509,
-                                      constants.HV_VNC_X509_VERIFY))
-
-    if hvparams[constants.HV_VNC_X509]:
-      if not os.path.isabs(hvparams[constants.HV_VNC_X509]):
-        raise errors.HypervisorError("The vnc x509 path must an absolute path"
-                                     ", if defined")
-
-    iso_path = hvparams[constants.HV_CDROM_IMAGE_PATH]
-    if iso_path and not os.path.isabs(iso_path):
-      raise errors.HypervisorError("The path to the CDROM image must be"
-                                   " an absolute path, if defined")
+    if (hvparams[constants.HV_VNC_X509_VERIFY] and
+        not hvparams[constants.HV_VNC_X509]):
+      raise errors.HypervisorError("%s must be defined, if %s is" %
+                                   (constants.HV_VNC_X509,
+                                    constants.HV_VNC_X509_VERIFY))
 
     boot_order = hvparams[constants.HV_BOOT_ORDER]
-    if boot_order not in constants.HT_KVM_VALID_BO_TYPES:
-      raise errors.HypervisorError(\
-        "The boot order must be one of %s" %
-        utils.CommaJoin(constants.HT_KVM_VALID_BO_TYPES))
-
-    if boot_order == constants.HT_BO_CDROM and not iso_path:
+    if (boot_order == constants.HT_BO_CDROM and
+        not hvparams[constants.HV_CDROM_IMAGE_PATH]):
       raise errors.HypervisorError("Cannot boot from cdrom without an"
                                    " ISO path")
 
-    nic_type = hvparams[constants.HV_NIC_TYPE]
-    if nic_type not in constants.HT_KVM_VALID_NIC_TYPES:
-      raise errors.HypervisorError(\
-        "Invalid NIC type %s specified for the KVM"
-        " hypervisor. Please choose one of: %s" %
-        (nic_type, utils.CommaJoin(constants.HT_KVM_VALID_NIC_TYPES)))
-    elif (boot_order == constants.HT_BO_NETWORK and
-          nic_type == constants.HT_NIC_PARAVIRTUAL):
-      raise errors.HypervisorError("Cannot boot from a paravirtual NIC. Please"
-                                   " change the NIC type.")
+    security_model = hvparams[constants.HV_SECURITY_MODEL]
+    if security_model == constants.HT_SM_USER:
+      if not hvparams[constants.HV_SECURITY_DOMAIN]:
+        raise errors.HypervisorError("A security domain (user to run kvm as)"
+                                     " must be specified")
+    elif (security_model == constants.HT_SM_NONE or
+          security_model == constants.HT_SM_POOL):
+      if hvparams[constants.HV_SECURITY_DOMAIN]:
+        raise errors.HypervisorError("Cannot have a security domain when the"
+                                     " security model is 'none' or 'pool'")
 
-    disk_type = hvparams[constants.HV_DISK_TYPE]
-    if disk_type not in constants.HT_KVM_VALID_DISK_TYPES:
-      raise errors.HypervisorError(\
-        "Invalid disk type %s specified for the KVM"
-        " hypervisor. Please choose one of: %s" %
-        (disk_type, utils.CommaJoin(constants.HT_KVM_VALID_DISK_TYPES)))
-
-    mouse_type = hvparams[constants.HV_USB_MOUSE]
-    if mouse_type and mouse_type not in constants.HT_KVM_VALID_MOUSE_TYPES:
-      raise errors.HypervisorError(\
-        "Invalid usb mouse type %s specified for the KVM hypervisor. Please"
-        " choose one of %s" %
-        utils.CommaJoin(constants.HT_KVM_VALID_MOUSE_TYPES))
-
-  def ValidateParameters(self, hvparams):
+  @classmethod
+  def ValidateParameters(cls, hvparams):
     """Check the given parameters for validity.
 
-    For the KVM hypervisor, this checks the existence of the
-    kernel.
+    @type hvparams:  dict
+    @param hvparams: dictionary with parameter names/value
+    @raise errors.HypervisorError: when a parameter is not valid
 
     """
-    super(KVMHypervisor, self).ValidateParameters(hvparams)
+    super(KVMHypervisor, cls).ValidateParameters(hvparams)
 
-    kernel_path = hvparams[constants.HV_KERNEL_PATH]
-    if kernel_path and not os.path.isfile(kernel_path):
-      raise errors.HypervisorError("Instance kernel '%s' not found or"
-                                   " not a file" % kernel_path)
-    initrd_path = hvparams[constants.HV_INITRD_PATH]
-    if initrd_path and not os.path.isfile(initrd_path):
-      raise errors.HypervisorError("Instance initrd '%s' not found or"
-                                   " not a file" % initrd_path)
+    security_model = hvparams[constants.HV_SECURITY_MODEL]
+    if security_model == constants.HT_SM_USER:
+      username = hvparams[constants.HV_SECURITY_DOMAIN]
+      try:
+        pwd.getpwnam(username)
+      except KeyError:
+        raise errors.HypervisorError("Unknown security domain user %s"
+                                     % username)
 
-    vnc_bind_address = hvparams[constants.HV_VNC_BIND_ADDRESS]
-    if vnc_bind_address and not utils.IsValidIP(vnc_bind_address) and \
-       not os.path.isdir(vnc_bind_address):
-       raise errors.HypervisorError("Instance vnc bind address must be either"
-                                    " an ip address or an existing directory")
+  @classmethod
+  def PowercycleNode(cls):
+    """KVM powercycle, just a wrapper over Linux powercycle.
 
-    vnc_x509 = hvparams[constants.HV_VNC_X509]
-    if vnc_x509 and not os.path.isdir(vnc_x509):
-      raise errors.HypervisorError("Instance vnc x509 path '%s' not found"
-                                   " or not a directory" % vnc_x509)
-
-    iso_path = hvparams[constants.HV_CDROM_IMAGE_PATH]
-    if iso_path and not os.path.isfile(iso_path):
-      raise errors.HypervisorError("Instance cdrom image '%s' not found or"
-                                   " not a file" % iso_path)
+    """
+    cls.LinuxPowercycle()
