@@ -52,7 +52,7 @@ def GetUserFiles(user, mkdir=False):
   if not user_dir:
     raise errors.OpExecError("Cannot resolve home of user %s" % user)
 
-  ssh_dir = os.path.join(user_dir, ".ssh")
+  ssh_dir = utils.PathJoin(user_dir, ".ssh")
   if not os.path.lexists(ssh_dir):
     if mkdir:
       try:
@@ -63,7 +63,7 @@ def GetUserFiles(user, mkdir=False):
   elif not os.path.isdir(ssh_dir):
     raise errors.OpExecError("path ~%s/.ssh is not a directory" % user)
 
-  return [os.path.join(ssh_dir, base)
+  return [utils.PathJoin(ssh_dir, base)
           for base in ["id_dsa", "id_dsa.pub", "authorized_keys"]]
 
 
@@ -75,7 +75,7 @@ class SshRunner:
     self.cluster_name = cluster_name
 
   def _BuildSshOptions(self, batch, ask_key, use_cluster_key,
-                       strict_host_check):
+                       strict_host_check, private_key=None, quiet=True):
     """Builds a list with needed SSH options.
 
     @param batch: same as ssh's batch option
@@ -84,6 +84,8 @@ class SshRunner:
     @param use_cluster_key: if True, use the cluster name as the
         HostKeyAlias name
     @param strict_host_check: this makes the host key checking strict
+    @param private_key: use this private key instead of the default
+    @param quiet: whether to enable -q to ssh
 
     @rtype: list
     @return: the list of options ready to use in L{utils.RunCmd}
@@ -94,11 +96,18 @@ class SshRunner:
       "-oHashKnownHosts=no",
       "-oGlobalKnownHostsFile=%s" % constants.SSH_KNOWN_HOSTS_FILE,
       "-oUserKnownHostsFile=/dev/null",
+      "-oCheckHostIp=no",
       ]
 
     if use_cluster_key:
       options.append("-oHostKeyAlias=%s" % self.cluster_name)
 
+    if quiet:
+      options.append("-q")
+
+    if private_key:
+      options.append("-i%s" % private_key)
+
     # TODO: Too many boolean options, maybe convert them to more descriptive
     # constants.
 
@@ -114,15 +123,21 @@ class SshRunner:
       else:
         options.append("-oStrictHostKeyChecking=no")
 
-    elif ask_key:
-      options.extend([
-        "-oStrictHostKeyChecking=ask",
-        ])
+    else:
+      # non-batch mode
+
+      if ask_key:
+        options.append("-oStrictHostKeyChecking=ask")
+      elif strict_host_check:
+        options.append("-oStrictHostKeyChecking=yes")
+      else:
+        options.append("-oStrictHostKeyChecking=no")
 
     return options
 
   def BuildCmd(self, hostname, user, command, batch=True, ask_key=False,
-               tty=False, use_cluster_key=True, strict_host_check=True):
+               tty=False, use_cluster_key=True, strict_host_check=True,
+               private_key=None, quiet=True):
     """Build an ssh command to execute a command on a remote node.
 
     @param hostname: the target host, string
@@ -135,13 +150,16 @@ class SshRunner:
     @param use_cluster_key: whether to expect and use the
         cluster-global SSH key
     @param strict_host_check: whether to check the host's SSH key at all
+    @param private_key: use this private key instead of the default
+    @param quiet: whether to enable -q to ssh
 
     @return: the ssh call to run 'command' on the remote host.
 
     """
-    argv = [constants.SSH, "-q"]
+    argv = [constants.SSH]
     argv.extend(self._BuildSshOptions(batch, ask_key, use_cluster_key,
-                                      strict_host_check))
+                                      strict_host_check, private_key,
+                                      quiet=quiet))
     if tty:
       argv.append("-t")
     argv.extend(["%s@%s" % (user, hostname), command])
@@ -179,7 +197,7 @@ class SshRunner:
       logging.error("File %s does not exist", filename)
       return False
 
-    command = [constants.SCP, "-q", "-p"]
+    command = [constants.SCP, "-p"]
     command.extend(self._BuildSshOptions(True, False, True, True))
     command.append(filename)
     command.append("%s:%s" % (node, filename))
@@ -212,7 +230,7 @@ class SshRunner:
         - detail: string with details
 
     """
-    retval = self.Run(node, 'root', 'hostname')
+    retval = self.Run(node, 'root', 'hostname --fqdn')
 
     if retval.failed:
       msg = "ssh problem"
@@ -221,7 +239,7 @@ class SshRunner:
         msg += ": %s" % output
       else:
         msg += ": %s (no output)" % retval.fail_reason
-      logging.error("Command %s failed: %s" % (retval.cmd, msg))
+      logging.error("Command %s failed: %s", retval.cmd, msg)
       return False, msg
 
     remotehostname = retval.stdout.strip()
